@@ -21,7 +21,7 @@
 #define AVFORMAT_AVIO_H
 
 /**
- * @file libavformat/avio.h
+ * @file
  * unbuffered I/O operations
  *
  * @warning This file has to be considered an internal but installed
@@ -31,6 +31,9 @@
 #include <stdint.h>
 
 #include "libavutil/common.h"
+#include "libavutil/log.h"
+
+#include "libavformat/version.h"
 
 /* unbuffered I/O */
 
@@ -42,7 +45,7 @@
  * sizeof(URLContext) must not be used outside libav*.
  */
 typedef struct URLContext {
-#if LIBAVFORMAT_VERSION_MAJOR >= 53
+#if FF_API_URL_CLASS
     const AVClass *av_class; ///< information for av_log(). Set by url_open().
 #endif
     struct URLProtocol *prot;
@@ -51,6 +54,7 @@ typedef struct URLContext {
     int max_packet_size;  /**< if non zero, the stream is packetized with this max packet size */
     void *priv_data;
     char *filename; /**< specified URL */
+    int is_connected;
 } URLContext;
 
 typedef struct URLPollEntry {
@@ -66,8 +70,8 @@ typedef struct URLPollEntry {
 typedef int URLInterruptCB(void);
 
 /**
- * Creates an URLContext for accessing to the resource indicated by
- * url, and opens it using the URLProtocol up.
+ * Create a URLContext for accessing to the resource indicated by
+ * url, and open it using the URLProtocol up.
  *
  * @param puc pointer to the location where, in case of success, the
  * function puts the pointer to the created URLContext
@@ -80,8 +84,26 @@ int url_open_protocol (URLContext **puc, struct URLProtocol *up,
                        const char *url, int flags);
 
 /**
- * Creates an URLContext for accessing to the resource indicated by
- * url, and opens it.
+ * Create a URLContext for accessing to the resource indicated by
+ * url, but do not initiate the connection yet.
+ *
+ * @param puc pointer to the location where, in case of success, the
+ * function puts the pointer to the created URLContext
+ * @param flags flags which control how the resource indicated by url
+ * is to be opened
+ * @return 0 in case of success, a negative value corresponding to an
+ * AVERROR code in case of failure
+ */
+int url_alloc(URLContext **h, const char *url, int flags);
+
+/**
+ * Connect an URLContext that has been allocated by url_alloc
+ */
+int url_connect(URLContext *h);
+
+/**
+ * Create an URLContext for accessing to the resource indicated by
+ * url, and open it.
  *
  * @param puc pointer to the location where, in case of success, the
  * function puts the pointer to the created URLContext
@@ -93,7 +115,7 @@ int url_open_protocol (URLContext **puc, struct URLProtocol *up,
 int url_open(URLContext **h, const char *url, int flags);
 
 /**
- * Reads up to size bytes from the resource accessed by h, and stores
+ * Read up to size bytes from the resource accessed by h, and store
  * the read bytes in buf.
  *
  * @return The number of bytes actually read, or a negative value
@@ -112,10 +134,24 @@ int url_read(URLContext *h, unsigned char *buf, int size);
  * certain there was either an error or the end of file was reached.
  */
 int url_read_complete(URLContext *h, unsigned char *buf, int size);
-int url_write(URLContext *h, unsigned char *buf, int size);
 
 /**
- * Changes the position that will be used by the next read/write
+ * Write size bytes from buf to the resource accessed by h.
+ *
+ * @return the number of bytes actually written, or a negative value
+ * corresponding to an AVERROR code in case of failure
+ */
+int url_write(URLContext *h, const unsigned char *buf, int size);
+
+/**
+ * Passing this as the "whence" parameter to a seek function causes it to
+ * return the filesize without seeking anywhere. Supporting this is optional.
+ * If it is not supported then the seek function will return <0.
+ */
+#define AVSEEK_SIZE 0x10000
+
+/**
+ * Change the position that will be used by the next read/write
  * operation on the resource accessed by h.
  *
  * @param pos specifies the new position to set
@@ -131,7 +167,7 @@ int url_write(URLContext *h, unsigned char *buf, int size);
 int64_t url_seek(URLContext *h, int64_t pos, int whence);
 
 /**
- * Closes the resource accessed by the URLContext h, and frees the
+ * Close the resource accessed by the URLContext h, and free the
  * memory used by it.
  *
  * @return a negative value if an error condition occurred, 0
@@ -140,17 +176,21 @@ int64_t url_seek(URLContext *h, int64_t pos, int whence);
 int url_close(URLContext *h);
 
 /**
- * Returns a non-zero value if the resource indicated by url
+ * Return a non-zero value if the resource indicated by url
  * exists, 0 otherwise.
  */
 int url_exist(const char *url);
 
+/**
+ * Return the filesize of the resource accessed by h, AVERROR(ENOSYS)
+ * if the operation is not supported by h, or another negative value
+ * corresponding to an AVERROR error code in case of failure.
+ */
 int64_t url_filesize(URLContext *h);
 
 /**
  * Return the file descriptor associated with this URL. For RTP, this
  * will return only the RTP file descriptor, not the RTCP file descriptor.
- * To get both, use rtp_get_file_handles().
  *
  * @return the file descriptor associated with this URL, or <0 on error.
  */
@@ -165,6 +205,12 @@ int url_get_file_handle(URLContext *h);
  * @return maximum packet size in bytes
  */
 int url_get_max_packet_size(URLContext *h);
+
+/**
+ * Copy the filename of the resource accessed by h to buf.
+ *
+ * @param buf_size size in bytes of buf
+ */
 void url_get_filename(URLContext *h, char *buf, int buf_size);
 
 /**
@@ -205,13 +251,6 @@ int av_url_read_pause(URLContext *h, int pause);
 int64_t av_url_read_seek(URLContext *h, int stream_index,
                          int64_t timestamp, int flags);
 
-/**
- * Passing this as the "whence" parameter to a seek function causes it to
- * return the filesize without seeking anywhere. Supporting this is optional.
- * If it is not supported then the seek function will return <0.
- */
-#define AVSEEK_SIZE 0x10000
-
 /**
  * Oring this flag as into the "whence" parameter to a seek function causes it to
  * seek by any means (like reopening and linear reading) or other normally unreasonble
@@ -224,7 +263,7 @@ typedef struct URLProtocol {
     const char *name;
     int (*url_open)(URLContext *h, const char *url, int flags);
     int (*url_read)(URLContext *h, unsigned char *buf, int size);
-    int (*url_write)(URLContext *h, unsigned char *buf, int size);
+    int (*url_write)(URLContext *h, const unsigned char *buf, int size);
     int64_t (*url_seek)(URLContext *h, int64_t pos, int whence);
     int (*url_close)(URLContext *h);
     struct URLProtocol *next;
@@ -232,9 +271,11 @@ typedef struct URLProtocol {
     int64_t (*url_read_seek)(URLContext *h, int stream_index,
                              int64_t timestamp, int flags);
     int (*url_get_file_handle)(URLContext *h);
+    int priv_data_size;
+    const AVClass *priv_data_class;
 } URLProtocol;
 
-#if LIBAVFORMAT_VERSION_MAJOR < 53
+#if FF_API_REGISTER_PROTOCOL
 extern URLProtocol *first_protocol;
 #endif
 
@@ -247,17 +288,24 @@ extern URLInterruptCB *url_interrupt_cb;
  */
 URLProtocol *av_protocol_next(URLProtocol *p);
 
-#if LIBAVFORMAT_VERSION_MAJOR < 53
+#if FF_API_REGISTER_PROTOCOL
 /**
  * @deprecated Use av_register_protocol() instead.
  */
 attribute_deprecated int register_protocol(URLProtocol *protocol);
+
+/**
+ * @deprecated Use av_register_protocol2() instead.
+ */
+attribute_deprecated int av_register_protocol(URLProtocol *protocol);
 #endif
 
 /**
- * Registers the URLProtocol protocol.
+ * Register the URLProtocol protocol.
+ *
+ * @param size the size of the URLProtocol struct referenced
  */
-int av_register_protocol(URLProtocol *protocol);
+int av_register_protocol2(URLProtocol *protocol, int size);
 
 /**
  * Bytestream IO Context.
@@ -307,6 +355,7 @@ ByteIOContext *av_alloc_put_byte(
                   int64_t (*seek)(void *opaque, int64_t offset, int whence));
 
 void put_byte(ByteIOContext *s, int b);
+void put_nbyte(ByteIOContext *s, int b, int count);
 void put_buffer(ByteIOContext *s, const unsigned char *buf, int size);
 void put_le64(ByteIOContext *s, uint64_t val);
 void put_be64(ByteIOContext *s, uint64_t val);
@@ -318,7 +367,21 @@ void put_le16(ByteIOContext *s, unsigned int val);
 void put_be16(ByteIOContext *s, unsigned int val);
 void put_tag(ByteIOContext *s, const char *tag);
 
-void put_strz(ByteIOContext *s, const char *buf);
+#if FF_API_OLD_AVIO
+attribute_deprecated void put_strz(ByteIOContext *s, const char *buf);
+#endif
+
+/**
+ * Write a NULL-terminated string.
+ * @return number of bytes written.
+ */
+int avio_put_str(ByteIOContext *s, const char *str);
+
+/**
+ * Convert an UTF-8 string to UTF-16LE and write it.
+ * @return number of bytes written.
+ */
+int avio_put_str16le(ByteIOContext *s, const char *str);
 
 /**
  * fseek() equivalent for ByteIOContext.
@@ -329,8 +392,9 @@ int64_t url_fseek(ByteIOContext *s, int64_t offset, int whence);
 /**
  * Skip given number of bytes forward.
  * @param offset number of bytes
+ * @return 0 on success, <0 on error
  */
-void url_fskip(ByteIOContext *s, int64_t offset);
+int url_fskip(ByteIOContext *s, int64_t offset);
 
 /**
  * ftell() equivalent for ByteIOContext.
@@ -339,7 +403,7 @@ void url_fskip(ByteIOContext *s, int64_t offset);
 int64_t url_ftell(ByteIOContext *s);
 
 /**
- * Gets the filesize.
+ * Get the filesize.
  * @return filesize or AVERROR
  */
 int64_t url_fsize(ByteIOContext *s);
@@ -375,13 +439,13 @@ void put_flush_packet(ByteIOContext *s);
 
 
 /**
- * Reads size bytes from ByteIOContext into buf.
+ * Read size bytes from ByteIOContext into buf.
  * @return number of bytes read or AVERROR
  */
 int get_buffer(ByteIOContext *s, unsigned char *buf, int size);
 
 /**
- * Reads size bytes from ByteIOContext into buf.
+ * Read size bytes from ByteIOContext into buf.
  * This reads at most 1 packet. If that is not enough fewer bytes will be
  * returned.
  * @return number of bytes read or AVERROR
@@ -410,7 +474,7 @@ static inline int url_is_streamed(ByteIOContext *s)
 }
 
 /**
- * Creates and initializes a ByteIOContext for accessing the
+ * Create and initialize a ByteIOContext for accessing the
  * resource referenced by the URLContext h.
  * @note When the URLContext h has been opened in read+write mode, the
  * ByteIOContext can be used only for writing.
@@ -424,7 +488,7 @@ int url_fdopen(ByteIOContext **s, URLContext *h);
 
 /** @warning must be called before any I/O */
 int url_setbufsize(ByteIOContext *s, int buf_size);
-#if LIBAVFORMAT_VERSION_MAJOR < 53
+#if FF_API_URL_RESETBUF
 /** Reset the buffer for reading or writing.
  * @note Will drop any data currently in the buffer without transmitting it.
  * @param flags URL_RDONLY to set up the buffer for reading, or URL_WRONLY
@@ -433,7 +497,22 @@ int url_resetbuf(ByteIOContext *s, int flags);
 #endif
 
 /**
- * Creates and initializes a ByteIOContext for accessing the
+ * Rewind the ByteIOContext using the specified buffer containing the first buf_size bytes of the file.
+ * Used after probing to avoid seeking.
+ * Joins buf and s->buffer, taking any overlap into consideration.
+ * @note s->buffer must overlap with buf or they can't be joined and the function fails
+ * @note This function is NOT part of the public API
+ *
+ * @param s The read-only ByteIOContext to rewind
+ * @param buf The probe buffer containing the first buf_size bytes of the file
+ * @param buf_size The size of buf
+ * @return 0 in case of success, a negative value corresponding to an
+ * AVERROR code in case of failure
+ */
+int ff_rewind_with_probe_data(ByteIOContext *s, unsigned char *buf, int buf_size);
+
+/**
+ * Create and initialize a ByteIOContext for accessing the
  * resource indicated by url.
  * @note When the resource indicated by url has been opened in
  * read+write mode, the ByteIOContext can be used only for writing.
@@ -486,7 +565,10 @@ int url_open_dyn_packet_buf(ByteIOContext **s, int max_packet_size);
 
 /**
  * Return the written size and a pointer to the buffer. The buffer
- *  must be freed with av_free().
+ * must be freed with av_free(). If the buffer is opened with
+ * url_open_dyn_buf, then padding of FF_INPUT_BUFFER_PADDING_SIZE is
+ * added; if opened with url_open_dyn_packet_buf, no padding is added.
+ *
  * @param s IO context
  * @param pbuffer pointer to a byte buffer
  * @return the length of the byte buffer
@@ -503,7 +585,7 @@ void init_checksum(ByteIOContext *s,
 /* udp.c */
 int udp_set_remote_url(URLContext *h, const char *uri);
 int udp_get_local_port(URLContext *h);
-#if (LIBAVFORMAT_VERSION_MAJOR <= 52)
+#if FF_API_UDP_GET_FILE
 int udp_get_file_handle(URLContext *h);
 #endif
 
